from typing import NamedTuple, List


class Item(NamedTuple):
    name: str
    weight: int
    value: float


def knapsack(items: List[Item], max_capacity: int) -> List[Item]:
    # построение таблицы динамического программирования
    table: List[List[float]] = [[0.0 for _ in range(max_capacity + 1)]
                                for _ in range(len(items) + 1)]
    for i, item in enumerate(items):
        for capacity in range(1, max_capacity + 1):
            previous_items_value: float = table[i][capacity]
            if capacity >= item.weight:  # #предмет помещается в рюкзак
                value_freeing_weight_for_item: float = table[i][capacity - item.weight]
                # только если этот предмет ценнее предыдущего
                table[i + 1][capacity] = max(value_freeing_weight_for_item + item.value,
                                             previous_items_value)
            else:  # для этого предмета нет места
                table[i + 1][capacity] = previous_items_value
    # найти решение в таблице
    solution: List[Item] = []
    capacity = max_capacity
    for i in range(len(items), 0, -1):  # идем в обратном направлении
        # этот предмет уже выбран?
        if table[i - 1][capacity] != table[i][capacity]:
            solution.append(items[i - 1])
            # если предмет выбран, то вычитаем его вес
            capacity -= items[i - 1].weight
    return solution


if __name__ == "__main__":
    items: List[Item] = [Item("телевизор", 50, 500),
                         Item("подсвечники", 2, 300),
                         Item("стереосистема", 35, 400),
                         Item("ноутбук", 3, 1000),
                         Item("еда", 15, 50),
                         Item("одежда", 20, 800),
                         Item("ювелирные изделия", 1, 4000),
                         Item("книги", 100, 300),
                         Item("принтер", 18, 30),
                         Item("холодильник", 200, 700),
                         Item("картина", 10, 1000)]
    print(knapsack(items, 75))
